<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\Compat;

use XF\Pub\App;

class AMPCompat
{
    public static function set_amp_page_params($reply, $templater, $url)
    {
        App::$allowPageCache = false;
        if (\XF::visitor()->user_id !== 0) {
            $templater->setPageParam('ampLoggedIn', $url);
            \XF::setVisitor(\XF::repository('XF:User')->getVisitor(0));
        }
        $templater->setPageParam('canonicalUrl', $url);
        $reply->setTemplateName('amp_' . $reply->getTemplateName());
        $templater->setPageParam('isAMP', true);
        if (\XF::options()->useFriendlyUrls) {
            $templater->setPageParam('pixelUrl', $url . '?amp-pixel=1');
        } else {
            $templater->setPageParam('pixelUrl', $url . '&amp-pixel=1');
        }
    }

    public static function set_meta_amp_url($templater, $url)
    {
        if (\XF::options()->ampHideFromTwitterbot && strpos(\XF::app()->request()->getUserAgent(), 'Twitterbot') !== false) {
            return;
        }
        if (\XF::options()->useFriendlyUrls) {
            $templater->setPageParam('ampUrl', $url . '?amp=1');
        } else {
            $templater->setPageParam('ampUrl', $url . '&amp=1');
        }
    }

    public static function addon_active($addon)
    {
        if (\XF::$versionId < 2020000) {
            return isset(\XF::app()->registry()['addOns'][$addon]);
        } else {
            return \XF::isAddOnActive($addon);
        }
    }

    public static function set_active_ad_positions($addon_id = null)
    {
        if ($addon_id and !in_array($addon_id, array('XenAddons/AMS', 'XenAddons/RMS'))) {
            return;
        }
        $finder = \XF::finder('XF:AdvertisingPosition');
        $ams_active = self::addon_active('XenAddons/AMS');
        $ams_positions = $finder->whereOr(
            ['position_id', 'ams_article_view_above_article_amp'],
            ['position_id', 'ams_article_view_below_article_amp']
        )->fetch();
        foreach ($ams_positions as $position) {
            if ($position->active != $ams_active) {
                $position->active = $ams_active;
                $position->save();
            }
        }

        $rms_active = self::addon_active('XenAddons/RMS');
        $rms_positions = $finder->whereOr(
            ['position_id', 'rms_item_view_above_item_details_content_amp'],
            ['position_id', 'rms_item_view_below_item_details_content_amp']
        )->fetch();
        foreach ($rms_positions as $position) {
            if ($position->active != $rms_active) {
                $position->active = $rms_active;
                $position->save();
            }
        }
    }

    public static function set_active_template_modifications($addon_id = null)
    {
        if ($addon_id and !in_array($addon_id, array('UW/FCS',))) {
            return;
        }
        $finder = \XF::finder('XF:TemplateModification');
        $uw_fcs_enabled = self::addon_active('UW/FCS');
        $uw_fcs_modification = $finder->where('modification_key', 'amp_uw_fcs_set_comments')->fetchOne();
        if ($uw_fcs_modification->enabled != $uw_fcs_enabled) {
            $uw_fcs_modification->enabled = $uw_fcs_enabled;
            $uw_fcs_modification->save();
        }
        if ($addon_id) {
            \XF::app()->simpleCache()->deleteSet('MaZ/AMP');
        }
    }
}