<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\Compat\XFMG\Pub\Controller;

use MaZ\AMP\Compat\AMPCompat;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\View;

class Media extends XFCP_Media
{
    public function actionIndex(ParameterBag $params)
    {
        if ($params->media_id && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'pixel', $params);
        }
        return parent::actionIndex($params);
    }

    public function actionView(ParameterBag $params)
    {
        $reply = parent::actionView($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $page = $this->filterPage($params->page);
            $mediaItem = $reply->getParam('mediaItem');
            $media_url = $this->buildLink('full:media', $mediaItem, ['page' => $page]);
            if ($this->request->filter('amp', 'str') == '1') {
                AMPCompat::set_amp_page_params($reply, $templater, $media_url);
            } else {
                AMPCompat::set_meta_amp_url($templater, $media_url);
            }
        }
        return $reply;
    }

    public function actionPixel(ParameterBag $params)
    {
        $visitor = \XF::visitor();
        $extraWith = ['Category'];
        if ($visitor->user_id)
        {
            $extraWith[] = 'Category.Permissions|' . $visitor->permission_combination_id;
            $extraWith[] = 'DraftComments|' . $visitor->user_id;
            $extraWith[] = 'CommentRead|' . $visitor->user_id;
            $extraWith[] = 'Viewed|' . $visitor->user_id;
            $extraWith[] = 'Reactions|' . $visitor->user_id;
            $extraWith[] = 'Watch|' . $visitor->user_id;
        }
        $this->assertViewableMediaItem($params->media_id, $extraWith);
        $cache = $this->app->simpleCache()->getSet('MaZ/AMP');
        $cache->setValue('pixel', $cache->getValue('pixel') + 1);
        return $this->view('MaZ\AMP\Pub\View\AMPPixelView', '');
    }
}