<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\Compat\XenAddons\RMS\Pub\Controller;

use MaZ\AMP\Compat\AMPCompat;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\View;

class Item extends XFCP_Item
{
    public function actionIndex(ParameterBag $params)
    {
        if ($params->item_id && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'pixel', $params);
        }
        return parent::actionIndex($params);
    }

    public function actionView(ParameterBag $params)
    {
        $reply = parent::actionView($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $item = $reply->getParam('item');
            $item_url = $this->buildLink('full:rms', $item);
            if ($this->request->filter('amp', 'str') == '1') {
                AMPCompat::set_amp_page_params($reply, $templater, $item_url);
            } else {
                AMPCompat::set_meta_amp_url($templater, $item_url);
            }
        }
        return $reply;
    }

    public function actionReviews(ParameterBag $params)
    {
        if ($params->item_id && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'pixel', $params);
        }
        $reply = parent::actionReviews($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $item = $reply->getParam('item');
            $page = $reply->getParam('page');
            $item_url = $this->buildPaginatedLink('full:rms/reviews', $item, $page);
            if ($this->request->filter('amp', 'str') == '1') {
                AMPCompat::set_amp_page_params($reply, $templater, $item_url);
            } else {
                AMPCompat::set_meta_amp_url($templater, $item_url);
            }
        }
        return $reply;
    }

    public function actionPixel(ParameterBag $params)
    {
        $this->assertViewableItem($params->item_id, $this->getItemViewExtraWith());
        $cache = $this->app->simpleCache()->getSet('MaZ/AMP');
        $cache->setValue('pixel', $cache->getValue('pixel') + 1);
        return $this->view('MaZ\AMP\Pub\View\AMPPixelView', '');
    }
}