<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/


namespace MaZ\AMP;

use MaZ\AMP\Compat\AMPCompat;
use XF;
use XF\Entity\Template;
use XF\Entity\StyleProperty;
use XF\Http\Response;
use XF\Mvc\Reply\AbstractReply;
use XF\Mvc\Reply\View;
use XF\Mvc\Entity\Entity;
use XF\Pub\App;

class Listener
{
    public static function entity_post_save(Entity $entity)
    {
        if ($entity instanceof StyleProperty) {
            if (isset($entity->group_name) && $entity->group_name == 'AMP') {
                XF::app()->simpleCache()->deleteSet('MaZ/AMP');
            }
            $watch = array('avatarBorderRadius',);
            if (in_array($entity->property_name, $watch)) {
                XF::app()->simpleCache()->deleteSet('MaZ/AMP');
            }
        }
        if ($entity instanceof Template && $entity->addon_id == 'MaZ/AMP') {
            XF::app()->simpleCache()->deleteSet('MaZ/AMP');
        }
    }

    private static function isAMP($app)
    {
        return $app->request()->filter('amp', 'str') == '1';
    }

    public static function app_setup(\XF\App $app)
    {
        if (\XF::$versionId < 2020000) {
            $templater_class = '\MaZ\AMP\XF\Template\XF21\Templater';
        } else {
            $templater_class = '\MaZ\AMP\XF\Template\XF22\Templater';
        }
        $app->extension()->addClassExtension('\XF\Template\Templater', $templater_class);

        if (AMPCompat::addon_active('XFMG')) {
            $cls = 'XFMG\Pub\Controller\Media';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XFRM')) {
            $cls = 'XFRM\Pub\Controller\ResourceItem';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XenAddons/AMS')) {
            $cls = 'XenAddons\AMS\Pub\Controller\ArticleItem';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XenAddons/CAS')) {
            $cls = 'XenAddons\CAS\Pub\Controller\Ad';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XenAddons/RMS')) {
            $cls = 'XenAddons\RMS\Pub\Controller\Item';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XenAddons/Showcase')) {
            $cls = 'XenAddons\Showcase\Pub\Controller\Item';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
        if (AMPCompat::addon_active('XenAddons/UBS')) {
            $cls = 'XenAddons\UBS\Pub\Controller\BlogEntryItem';
            $cls2 = 'XenAddons\UBS\Pub\Controller\BlogEntryPage';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
            $app->extension()->addClassExtension($cls2, 'MaZ\AMP\Compat\\' . $cls2);
        }
        if (AMPCompat::addon_active('EWR/Porta')) {
            $cls = 'EWR\Porta\Pub\Controller\Article';
            $app->extension()->addClassExtension($cls, 'MaZ\AMP\Compat\\' . $cls);
        }
    }

    public static function app_complete(XF\App $app, Response $response)
    {
        if ($response->httpCode() === 200 && self::isAMP($app)) {
            $html = $response->body();
            if (strpos($html, '<html ⚡') !== false) {
                if (strpos($html, 'data-amp-page-type="thread"') !== false || strpos($html, 'data-amp-page-type=""') !== false) {
                    Styles::insertCss($app, 'thread', self::style_templates(), $html);
                } else if (strpos($html, 'data-amp-page-type="forum"') !== false || strpos($html, 'data-amp-page-type="xenporta"') !== false) {
                    $extra = ['public:ampnode_list.less'];
                    if (AMPCompat::addon_active('EWR/Porta')) {
                        array_push($extra, 'public:EWRporta.less', 'public:EWRporta_articles.less', 'public:widget_EWRporta_features.less');
                    }
                    Styles::insertCss($app, 'forums', self::style_templates($extra), $html);
                } else if (strpos($html, 'data-amp-page-type="resource"') !== false) {
                    Styles::insertCss($app, 'resource', self::style_templates(['public:xfrm.less']), $html);
                } else if (strpos($html, 'data-amp-page-type="media"') !== false) {
                    $styles = self::style_templates(['public:core_xfmg.less', 'public:amp_xfmg_media_view.less']);
                    Styles::insertCss($app, 'resource', $styles, $html);
                } else if (strpos($html, 'data-amp-page-type="xa_ams"') !== false) {
                    $styles = self::style_templates(['public:core_xa_ams.less', 'public:amp_xa_ams_review.less', 'public:xa_ams.less']);
                    Styles::insertCss($app, 'xa_ams', $styles, $html);
                } else if (strpos($html, 'data-amp-page-type="xa_cas"') !== false) {
                    $styles = self::style_templates(['public:core_xa_cas.less', 'public:xa_cas.less']);
                    Styles::insertCss($app, 'xa_ams', $styles, $html);
                } else if (strpos($html, 'data-amp-page-type="xa_ubs"') !== false) {
                    $styles = self::style_templates(['public:core_xa_ubs.less', 'public:amp_xa_ubs_review.less', 'public:xa_ubs.less']);
                    Styles::insertCss($app, 'xa_ubs', $styles, $html);
                } else if (strpos($html, 'data-amp-page-type="xa_rms"') !== false) {
                    $styles = self::style_templates(['public:core_xa_rms.less', 'public:amp_xa_rms_review.less', 'public:xa_rms.less']);
                    Styles::insertCss($app, 'xa_rms', $styles, $html);
                } else if (strpos($html, 'data-amp-page-type="xa_sc"') !== false) {
                    $styles = self::style_templates(['public:core_xa_sc.less', 'public:amp_xa_sc_review.less', 'public:xa_sc.less']);
                    Styles::insertCss($app, 'xa_sc', $styles, $html);
                }
                $html = Markup::cleanup($html);
                $html = Styles::insertSprites($app, $html);
                $html = Styles::minimizeCSS($html);
                $response->body($html);
            }
        }
    }

    public static function app_pub_render_page(App $app, array &$params, AbstractReply $reply)
    {
        if ($reply instanceof View) {
            if (self::isAMP($app)) {
                $view_class = $reply->getViewClass();
                if (strpos($view_class, 'XF:Thread\View') !== false) {
                    $params['ampPageType'] = 'thread';
                }
                if (strpos($view_class, 'XF:Forum\\') !== false) {
                    $params['ampPageType'] = 'forum';
                }
                if (strpos($view_class, 'XFMG:Media') !== false) {
                    $params['ampPageType'] = 'media';
                }
                if (strpos($view_class, 'XFRM:ResourceItem') !== false) {
                    $params['ampPageType'] = 'resource';
                }
                if (strpos($view_class, 'XenAddons\AMS:ArticleItem\View') !== false) {
                    $params['ampPageType'] = 'xa_ams';
                }
                if (strpos($view_class, 'XenAddons\CAS:Ad\View') !== false) {
                    $params['ampPageType'] = 'xa_cas';
                }
                if (strpos($view_class, 'XenAddons\UBS:BlogEntry') !== false) {
                    $params['ampPageType'] = 'xa_ubs';
                }
                if (strpos($view_class, 'XenAddons\RMS') !== false) {
                    $params['ampPageType'] = 'xa_rms';
                }
                if (strpos($view_class, 'XenAddons\Showcase') !== false) {
                    $params['ampPageType'] = 'xa_sc';
                }
                if (strpos($view_class, 'EWR\Porta:Articles') !== false) {
                    $params['ampPageType'] = 'xenporta';
                }
                $params['ampGAId'] = XF::options()->googleAnalyticsWebPropertyId;
                if (XF::options()->ampGAWebPropertyId) {
                    $params['ampGAId'] = XF::options()->ampGAWebPropertyId;
                }
                $params['ampVersion'] = $app->container('addon.cache')['MaZ/AMP'];
                $params['debugging'] = XF::$debugMode;

                $params['ampHTMLClass'] = '';
                if (isset($app->templater()->pageParams['ampLoggedIn'])) {
                    $params['ampHTMLClass'] .= ' show-broken-links';
                }
                if (XF::options()->ampThemeHouseFixFontSize) {
                    $params['ampHTMLClass'] .= ' TH-fontfix';
                }
                $params['ampLogo'] = Styles::getLogo($app);
            }
        }
    }

    public static function style_templates(array $extra_styles = null)
    {
        $styles = array(
            'public:amp_fa.css',
            'public:ampcore.less',
            'public:ampapp.less',
            'public:ampmessage.less',
            'public:ampattachments.less',
            'public:amppoll.less',
            'public:ampnotices.less',
            'public:ampstructured_list.less',
            'public:amprating_stars.less',
            'public:amp_override.less',
            'public:amp_style_compatibility.less',
        );
        if ($extra_styles) {
            $styles = array_merge($styles, $extra_styles);
        }
        array_push($styles, 'public:amp_extra.less');
        return $styles;
    }


    public static function templater_macro_pre_render_admin(\XF\Template\Templater $templater, &$type, &$template, &$name, array &$arguments, array &$globalVars)
    {
        if ($arguments['group'] && $arguments['group']->group_id == 'AMP') {
            $template = 'maz_amp_option_macros';
            $name = 'option_form_block_tabs';
            $arguments['headers'] = [
                'generalOptions' => [
                    'label' => \XF::phrase('general_options'),
                    'minDisplayOrder' => 0,
                    'maxDisplayOrder' => 500,
                    'active' => true
                ],
                'adsAnalyticOptions' => [
                    'label' => \XF::phrase('advertising_and_analytics'),
                    'minDisplayOrder' => 500,
                    'maxDisplayOrder' => 1000
                ],
                'advancedOptions' => [
                    'label' => \XF::phrase('advanced_options'),
                    'minDisplayOrder' => 1000,
                    'maxDisplayOrder' => -1
                ],
            ];
        }
    }

    public static function templater_template_pre_render(\XF\Template\Templater $templater, &$type, &$template, array &$params)
    {
        $params['isAMP'] = \XF::app()->request()->filter('amp', 'str') == '1';
    }

    public static function templater_macro_pre_render(\XF\Template\Templater $templater, &$type, &$template, &$name, array &$arguments, array &$globalVars)
    {
        $arguments['isAMP'] = false;
        if (isset($globalVars['isAMP'])) {
            $arguments['isAMP'] = $globalVars['isAMP'];
        }
    }

    public static function templater_global_data(\XF\App $app, array &$data, $reply)
    {
        if ($app->container('app.classType') == 'Admin' && $app->request()->getRoutePath() == '') {
            if (!\XF::options()->ampHideUpdateAvailable) {
                $new_version = $app->simpleCache()->getValue('MaZ/AMP', 'newVersion');
                if (isset($new_version) && $new_version > $app->container('addon.cache')['MaZ/AMP']) {
                    $data['ampNewVersionLink'] = $app->simpleCache()->getValue('MaZ/AMP', 'newVersionLink') . '?ref=' . urlencode(\XF::options()->boardUrl);
                }
            }
        }
        $data['isAMP'] = $app->request()->filter('amp', 'str') == '1';
    }

    public static function criteria_page($rule, array $data, \XF\Entity\User $user, array $params, &$returnValue)
    {
        $amp_template = false;
        if (isset($params['template'])) {
            $amp_template = strpos($params['template'], 'amp_') === 0;
        }
        switch ($rule) {
            case 'amp_page':
                if ($data['limit'] === 'not_amp') {
                    $returnValue = !$amp_template;
                }
                if ($data['limit'] === 'only_amp') {
                    $returnValue = $amp_template;
                }
                break;
        }
    }

    public static function addon_post_install(\XF\AddOn\AddOn $addOn, \XF\Entity\AddOn $installedAddOn, array $json, array &$stateChanges)
    {
        AMPCompat::set_active_ad_positions($installedAddOn->addon_id);
        AMPCompat::set_active_template_modifications($installedAddOn->addon_id);
    }

    public static function addon_post_uninstall(\XF\AddOn\AddOn $addOn, $addOnId, array $json)
    {
        AMPCompat::set_active_ad_positions($addOnId);
        AMPCompat::set_active_template_modifications($addOnId);
    }
}