<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP;

use DOMDocument;

class Markup
{
    public static function cleanup($html)
    {
        $options = \XF::options();
        $html = strtr($html, [
            ' target=""' => ' ',
            ' target="blank"' => ' target="_blank"',
            ' target="_blank "' => ' target="_blank"',
            ' href="javascript:' => ' href="#javascript:',
            '<IMG ' => '<img ',
            ' uix-href="' => ' data-uix-href="',
            ' uix-data-href="' => ' data-uix-href="',
            ' mega-menu-target="' => ' data-mega-menu-target="',
            ' border="0"' => '',
            '<i class="fad ' => '<i class="fas ', // Duotone icons rarely used, and add extra assets to load.. change to solid
        ]);
        if (strpos($html, '<script class="') !== false) {
            $html = preg_replace('#<script class.+?</script>#s', '', $html);
        }
        if (strpos($html, '<script type="text/javascript"') !== false) {
            $html = preg_replace('#<script type="text/javascript".+?</script>#s', '', $html);
        }
        if (strpos($html, '<script type="text/template"') !== false) {
            $html = preg_replace('#<script type="text/template".+?</script>#s', '', $html);
        }
        if (strpos($html, '<script>') !== false) {
            $html = preg_replace('#<script>.+?</script>#s', '', $html);
        }
        if (strpos($html, '<style>') !== false) {
            $html = preg_replace('#<style>.+?</style>#s', '', $html);
        }
        if (strpos($html, '<div class="samBannerUnit') !== false) {
            $html = preg_replace_callback('#<div class="samBannerUnit.*?>#s', function ($match) {
                return preg_replace('#style=".*?"#', '', $match[0]);
            }, $html);
        }
        $html = preg_replace_callback('#<body.*?>.+?</body>#s', array(__CLASS__, 'bodyHandler',), $html);
        if (strpos($html, '<img') !== false) {
            $html = preg_replace_callback('#<img\s.+?>#s', array(__CLASS__, 'imgHandler',), $html);
        }
        if (strpos($html, 'lazy') !== false) {
            $html = preg_replace_callback('#<amp-img.+?>#s', array(__CLASS__, 'removeImgLazyLoad',), $html);
        }
        if (strpos($html, '<form') !== false) {
            $html = preg_replace_callback('#<form\s.+?</form>#s', array(__CLASS__, 'formHandler',), $html);
            if (strpos($html, 'https://cdn.ampproject.org/v0/amp-form-0.1.js') === false) {
                $html = str_replace('</head>', '<script async custom-element="amp-form" src="https://cdn.ampproject.org/v0/amp-form-0.1.js"></script></head>', $html);
            }
        }
        if (strpos($html, '<iframe') !== false) {
            $html = preg_replace_callback('#<iframe\s.+?</iframe>#s', array(__CLASS__, 'iframeHandler',), $html);
        }
        if (strpos($html, '<object') !== false) {
            $html = preg_replace_callback('#<object\s.+?</object>#s', array(__CLASS__, 'objectHandler',), $html);
        }
        if (strpos($html, '<video') !== false) {
            $html = preg_replace_callback('#<video\s.+?</video>#s', array(__CLASS__, 'videoHandler',), $html);
        }
        if (strpos($html, '<audio') !== false) {
            $html = preg_replace_callback('#<audio\s.+?</audio>#s', array(__CLASS__, 'audioHandler',), $html);
        }
        if (strpos($html, '!important') !== false) {
            $html = preg_replace_callback('#style=".+?"#s', array(__CLASS__, 'styleImportantHandler'), $html);
        }
        if (strpos($html, 'data-s9e-mediaembed') !== false) {
            $html = preg_replace_callback('#<span data-s9e-mediaembed.+?</span>#s', function ($matching) {
                return preg_replace('# style=".+?"#', '', $matching[0]);
            }, $html);
        }
        if ($options->ampMathMLConvert) {
            $html = preg_replace_callback('#<article.+?</article>#s', array(__CLASS__, 'mathMLHandler',), $html);
        }
        $html = self::setHeadScripts($html);
        return $html;
    }

    protected static function setHeadScripts($html)
    {
        $scripts = [
            ['<amp-social-share ', 'amp-social-share', 'https://cdn.ampproject.org/v0/amp-social-share-0.1.js'],
            ['<amp-iframe ', 'amp-iframe', 'https://cdn.ampproject.org/v0/amp-iframe-0.1.js'],
            ['<amp-ad', 'amp-ad', 'https://cdn.ampproject.org/v0/amp-ad-0.1.js'],
            ['<amp-auto-ads ', 'amp-auto-ads', 'https://cdn.ampproject.org/v0/amp-auto-ads-0.1.js'],
            ['<amp-sticky-ad ', 'amp-sticky-ad', 'https://cdn.ampproject.org/v0/amp-sticky-ad-1.0.js'],
            ['<amp-video ', 'amp-video', 'https://cdn.ampproject.org/v0/amp-video-0.1.js'],
            ['<amp-audio ', 'amp-audio', 'https://cdn.ampproject.org/v0/amp-audio-0.1.js'],
            ['<amp-mathml ', 'amp-mathml', 'https://cdn.ampproject.org/v0/amp-mathml-0.1.js'],
            ['<amp-skimlinks ', 'amp-skimlinks', 'https://cdn.ampproject.org/v0/amp-skimlinks-0.1.js'],
            ['<amp-link-rewriter ', 'amp-link-rewriter', 'https://cdn.ampproject.org/v0/amp-link-rewriter-0.1.js'],
            ['<amp-google-document-embed ', 'amp-google-document-embed', 'https://cdn.ampproject.org/v0/amp-google-document-embed-0.1.js'],
        ];
        foreach ($scripts as list($find, $el, $src)) {
            if (strpos($html, $find) !== false && strpos($html, $src) === false) {
                $html = str_replace('</head>', '<script async custom-element="' . $el . '" src="' . $src . '"></script></head>', $html);
            }
        }
        return $html;
    }

    protected static function mathMLHandler($matching)
    {
        $html = $matching[0];
        $html = preg_replace_callback('#<code.+?</code>#s', array(__CLASS__, 'mathMLCodeProtectHandler',), $html);
        $html = preg_replace_callback('#(title|alt|author|desc)=".+?"#s', array(__CLASS__, 'mathMLCodeProtectHandler',), $html);
        $html = strtr($html, [
            '\\(' => '<amp-mathml layout="container" inline data-formula="\\(',
            '\\)' => '\\)"></amp-mathml>',
            '\\[' => '<amp-mathml layout="container" data-formula="\\[',
            '\\]' => '\\]"></amp-mathml>',
        ]);
        if (strpos($html, '$') !== false) {
            $html = preg_replace_callback('#[\$]{1,2}.*?[\$]{1,2}#s', array(__CLASS__, 'mathMLParagraphHandler',), $html);
        }
        $html = preg_replace_callback('#<amp-mathml.*?</amp-mathml>#s', function ($match) {
            return str_replace('<br />', '', $match[0]);
        }, $html);
        return strtr($html, [
            'MAZAMPDOLLAR' => '$',
            'MAZAMPINLINEOPEN' => '\\(',
            'MAZAMPINLINECLOSE' => '\\)',
            'MAZAMPDISPLAYOPEN' => '\\[',
            'MAZAMPDISPLAYCLOSE' => '\\]',
        ]);
    }

    protected static function mathMLCodeProtectHandler($matching)
    {
        return strtr($matching[0], [
            '$' => 'MAZAMPDOLLAR',
            '\\(' => 'MAZAMPINLINEOPEN',
            '\\)' => 'MAZAMPINLINECLOSE',
            '\\[' => 'MAZAMPDISPLAYOPEN',
            '\\]' => 'MAZAMPDISPLAYCLOSE',
        ]);
    }

    protected static function mathMLParagraphHandler($matching)
    {
        $length = strlen($matching[0]);
        if (strpos($matching[0], '$$') === 0) {
            if ($matching[0][$length - 1] == '$' && $matching[0][$length - 2] == '$') {
                return '<amp-mathml layout="container" data-formula="' . $matching[0] . '"></amp-mathml>';
            }
            return $matching[0];
        }
        if ($matching[0][$length - 1] == '$' && $matching[0][$length - 2] != '$') {
            if (strpos($matching[0], '$$') !== false) {
                return $matching[0];
            }
            $formula = str_replace('$', '', $matching[0]);
            return '<amp-mathml layout="container" inline data-formula="\(' . $formula . '\)"></amp-mathml>';
        }
        return $matching[0];
    }

    protected static function bodyHandler(array $matching)
    {
        $html = preg_replace_callback('#<script(.*?)>.*?</script>#s', function ($matches) {
            if (strpos($matches[1], ' src=') !== false) {
                return '';
            }
            if (strpos($matches[1], 'text/javascript') !== false) {
                return '';
            }
            return $matches[0];
        }, $matching[0]);
        $html = preg_replace('# on(blur|click|load|input|submit)=".*?"#i', '', $html);
        $html = preg_replace_callback('# href="(.*?)"#s', array(__CLASS__, 'URLHandler'), $html);
        return $html;
    }

    protected static function formHandler(array $matching)
    {
        return str_replace(' action="', ' action-xhr="', $matching[0]);
    }

    protected static function URLHandler(array $href_match)
    {
        if ($href_match[1] == '' || $href_match[1][0] == '/' || $href_match[1][0] == '#' || $href_match[1] == 'reply') { // Ignore relative paths
            return $href_match[0];
        }
        if (strpos($href_match[1], '../') === 0) {
            return $href_match[0];
        }
        if (preg_match('#(://tel:|://mailto:)#si', $href_match[1])) { // Try tp cleanup broken tel:/mailto: urls
            $href_match[1] = strtr($href_match[1], ['http://' => '', 'https://' => '']);
        }
        preg_match('#\[url=(.*?)(\]|$)#si', $href_match[1], $bb_url_match);
        if (isset($bb_url_match[1])) {
            $href_match[1] = $bb_url_match[1];
        }
        preg_match('#\[url\](.*?)(\[|$)#si', $href_match[1], $bb_url_match);
        if (isset($bb_url_match[1])) {
            $href_match[1] = $bb_url_match[1];
        }
        if (!self::validate_rfc2396_url($href_match[1])) {
            return str_replace('href="', 'href="#amp-broken-url:', $href_match[0]);
        }
        return ' href="' . $href_match[1] . '"';
    }

    private static function validate_rfc2396_url($url)
    {
        if ($url == '') { // Empty links are "okay" (although probably not intentional)
            return true;
        }
        $url = utf8_deaccent(utf8_romanize($url)); // Romanize special characters to be valid rfc2396..
        $url = preg_replace('#[\p{Han}\p{Hiragana}\p{Katakana}]#u', 'a', $url); // Japanese characters
        if (strpos($url, 'tel:') === 0) {
            // FILTER_VALIDATE_URL is bad with "tel:" (https://www.php.net/manual/en/filter.filters.validate.php)
            return true;
        }
        $host = parse_url($url, PHP_URL_HOST);
        if (substr_count($host, '.') > 1) {
            $subdomain = explode('.', $host)[0];
            $url = str_replace($subdomain, str_replace('_', '-', $subdomain), $url);
        }
        $path = parse_url($url, PHP_URL_PATH);
        $query = parse_url($url, PHP_URL_QUERY);
        $hash = parse_url($url, PHP_URL_FRAGMENT);
        $url = str_replace($path, implode('/', array_map('urlencode', explode('/', $path))), $url);
        $url = str_replace($query, implode('/', array_map('urlencode', explode('/', $query))), $url);
        $url = str_replace($hash, implode('/', array_map('urlencode', explode('/', $hash))), $url);
        return filter_var($url, FILTER_VALIDATE_URL) ? true : false;
    }

    protected static function videoHandler(array $matching)
    {
        $html = str_replace('<video ', '<amp-video layout="responsive" ', $matching[0]);
        $html = str_replace('</video>', '</amp-video>', $html);
        if (strpos($html, ' height="') === false) {
            $html = str_replace('<amp-video ', '<amp-video height="200" ', $html);
        }
        if (strpos($html, ' width="') === false) {
            $html = str_replace('<amp-video ', '<amp-video width="300" ', $html);
        }
        return $html;
    }

    protected static function audioHandler(array $matching)
    {
        $html = str_replace('<audio ', '<amp-audio ', $matching[0]);
        $html = str_replace('</audio>', '</amp-audio>', $html);
        if (strpos($html, ' height="') === false) {
            $html = str_replace('<amp-audio ', '<amp-audio height="50"', $html);
        }
        if (strpos($html, ' width="') === false) {
            $html = str_replace('<amp-audio ', '<amp-audio width="auto"', $html);
        }
        return $html;
    }

    protected static function styleImportantHandler(array $matching)
    {
        return str_replace('!important', '', $matching[0]);
    }

    protected static function samCodeBanner(array $matching)
    {

        return $matching[0];
    }

    protected static function objectHandler($matching)
    {
        $disable_errors = libxml_use_internal_errors(true);
        $dom = new DOMDocument();
        $dom->loadHTML($matching[0]);
        libxml_use_internal_errors($disable_errors);
        if (!$dom) {
            return '<!-- AMPRobot warning: Broken object code omitted: ' . $matching[0] . '--->';
        }
        $object = $dom->getElementsByTagName('object')->item(0);
        $url = $object->getAttribute('data');
        if (strpos($url, '//') === 0) {
            $url = 'https:' . $url;
        }
        if (strpos($url, '/') === 0) {
            $url = \XF::options()->boardUrl . $url;
        }
        if (strpos($url, 'http') !== 0) {
            return '<!-- AMPRobot warning: Broken object code omitted: ' . $matching[0] . '--->';
        }
        $type = $object->getAttribute('type');
        if (empty($type) || $type != 'application/pdf') {
            return '<a href="' . $url . '">' . $url . '</a>';
        }
        return '<amp-google-document-embed src="' . $url . '" width="100" height="100" layout="responsive"></amp-google-document-embed>';
    }

    protected static function iframeHandler(array $matching)
    {
        $disable_errors = libxml_use_internal_errors(true);
        $dom = new DOMDocument();
        $dom->loadHTML($matching[0]);
        libxml_use_internal_errors($disable_errors);
        if (!$dom) {
            return '<!-- AMPRobot warning: Broken iframe code omitted: ' . $matching[0] . '--->';
        }
        $iframe = $dom->getElementsByTagName('iframe')->item(0);
        $src = $iframe->getAttribute('src');
        if (strpos($src, '//') === 0) {
            $src = 'https:' . $src;
        }
        if (strpos($src, 'https://') !== 0) {
            return '<!-- AMPRobot warning: <amp-iframe> only supports HTTPS src ---><a href="' . $src . '">' . $src . '</a>';
        }
        $copy_attrs = array('srcdoc', 'frameborder', 'allowfullscreen', 'allowpaymentrequest', 'allowtransparency', 'referrerpolicy', 'scrolling');
        $attrs = 'sandbox="allow-scripts allow-popups" ';
        foreach ($copy_attrs as $attr) {
            if (strpos($matching[0], $attr) !== false) {
                $val = $iframe->getAttribute($attr);
                if (strpos($attr, 'allow') === 0) {
                    $val = '';
                }
                $attrs = $attrs . $attr . '="' . $val . '" ';
            }
        }
        $attrs = str_replace('scrolling=""', '', $attrs); // Not allowed in AMP spec
        $layout = 'responsive';
        $width = $iframe->getAttribute('width');
        $height = $iframe->getAttribute('height');
        $style = $iframe->getAttribute('style');
        if (!$height && $style) {
            $matches = array();
            preg_match('#height:\s*(\d+)[vp%";]#', $style, $matches);
            if ($matches && !empty($matches[1])) {
                $height = $matches[1];
            }
        }
        if (!$width && $style) {
            $matches = array();
            preg_match('#width:\s*(\d+)[vp%";]#', $style, $matches);
            if ($matches && !empty($matches[1])) {
                $width = $matches[1];
            }
        }
        if (!$width) {
            $width = 100;
        }
        if (!$height) {
            $height = 50;
        }
        if ($width == '100%' && strpos($height, '%') === false) {
            if ($height > 100) {
                $height = 50;
            }
        }
        $width = str_replace('%', '', $width);
        $height = str_replace('%', '', $height);
        return '<amp-iframe src="' . $src . '" layout="' . $layout . '" height="' . $height . '" width="' . $width . '" ' . $attrs . '></amp-iframe>';
    }

    protected static function imgHandler(array $matching)
    {
        $disable_errors = libxml_use_internal_errors(true);
        $dom = new DOMDocument();
        $dom->loadHTML($matching[0]);
        libxml_use_internal_errors($disable_errors);
        if (!$dom) {
            return '<!--- AMPRobot warning: Broken img code omitted: ' . $matching[0] . '--->';
        }
        $img = $dom->getElementsByTagName('img')->item(0);
        $src = $img->getAttribute('src');
        if (strpos($src, 'clear.png') !== false) {
            return '<!--- AMPRobot debug: hiding transparent image ---->';
        }
        $width = $img->getAttribute('width');
        $height = $img->getAttribute('height');
        $class = $img->getAttribute('class');
        $layout = 'intrinsic';
        if (!$width && !$height && strpos($class, 'smilie smilie--emoji') !== false) {
            $width = 22;
            $height = 22;
            $layout = 'fixed';
        }
        if ($width && $height) {
            return '<amp-img src="' . $src . '" height="' . $height . '" width="' . $width . '" layout="' . $layout . '" class="' . $class . '"></amp-img>';
        }
        if ($src == '' && strpos($matching[0], ' data-src="') !== false) { // Siropu lazy loading
            $matching[0] = str_replace(' data-src="', ' src="', $matching[0]);
        }
        $matching[0] = preg_replace('# (align|border|hspace|vspace|width|height)=".*?"#i', '', $matching[0]);
        return preg_replace('#<img\s(.+?)>#s', '<div class="unknown-img"><amp-img layout="fill" $1></amp-img></div>', $matching[0]);
    }

    protected static function removeImgLazyLoad(array $matching)
    {
        $matching[0] = str_replace('loading="lazy"', '', $matching[0]);
        $matching[0] = str_replace("loading='lazy'", '', $matching[0]);
        if (strpos($matching[0], 'lazy') !== false) {
            $matching[0] = preg_replace_callback('# class="(.+?)"#s', array(__CLASS__, 'removeLazyClass',), $matching[0]);
        }
        return $matching[0];
    }

    protected static function removeLazyClass(array $matching)
    {
        return str_replace('lazy', '', $matching[0]);
    }

}