<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\Pub\Controller;

use GuzzleHttp\Exception\RequestException;
use MaZ\AMP\Compat\AMPCompat;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class AMPRobot extends AbstractController
{
    public function actionIndex(ParameterBag $params)
    {
        if (\XF::visitor()->user_id !== 0) {
            \XF::setVisitor(\XF::repository('XF:User')->getVisitor(0));
        }
        $app = $this->app();
        $options = \XF::options();
        $visitor = \XF::visitor();
        $style = $app->style($visitor->style_id);
        $logo_url = $style->getProperty('publicLogoUrl');
        $logo_width = $style->getProperty('publicLogoWidth');
        $logo_height = $style->getProperty('publicLogoHeight');
        $amp_logo = $style->getProperty('ampCustomLogo');
        if ($amp_logo) {
            $logo_url = $amp_logo;
            $logo_width = $style->getProperty('ampLogoWidth');
            $logo_height = $style->getProperty('ampLogoHeight');
        }
        if (isset($logo_url) && $logo_url && $logo_url[0] == '/') {
            $logo_url = substr($logo_url, 1);
        }
        $header = array_merge($style->getProperty('publicHeader'), $style->getProperty('ampHeader'));
        $pather = $app->container('request.pather');
        if (isset($header['background-image'])) {
            $header_bg_img = substr($header['background-image'], 5, -2);
            if ($header_bg_img[0] == '/') {
                $header_bg_img = substr($header_bg_img, 1);
            }
            $header['background-image'] = 'url(' . $pather($header_bg_img, 'full') . ')';
        }
        $viewParams = [
            'site_name' => $options->boardTitle,
            'logo_url' => $pather($logo_url ?: '', 'full'),
            'logo_width' => $logo_width,
            'logo_height' => $logo_height,
            'logo_enabled' => boolval($options->ampShowLogo),
            'logo_text' => boolval($options->ampTextLogo),
            'friendly_urls' => boolval($options->useFriendlyUrls),
            'root_url' => $options->boardUrl,
            'ampbot_url' => $this->buildLink('full:amprobot'),
            'threads_url' => $this->buildLink('full:threads'),
            'sitemap_url' => $this->buildLink('full:sitemap.xml'),
            'version' => $app->container('addon.cache')['MaZ/AMP'],
            'header' => $header,
        ];
        if (!\XF::options()->useFriendlyUrls) {
            $viewParams['sitemap_url'] = $viewParams['root_url'] . '/sitemap.php';
        }
        if (AMPCompat::addon_active('XFMG')) {
            $viewParams['xfmg_url'] = $this->buildLink('full:media');
        }
        if (AMPCompat::addon_active('XFRM')) {
            $viewParams['xfrm_url'] = $this->buildLink('full:resources');
        }
        if (AMPCompat::addon_active('XenAddons/AMS')) {
            $viewParams['xa_ams_url'] = $this->buildLink('full:ams');
        }
        if (AMPCompat::addon_active('XenAddons/RMS')) {
            $viewParams['xa_rms_url'] = $this->buildLink('full:rms');
        }
        if (AMPCompat::addon_active('XenAddons/Showcase')) {
            $viewParams['xa_sc_url'] = $this->buildLink('full:showcase');
        }
        if (AMPCompat::addon_active('XenAddons/UBS')) {
            $viewParams['xa_ubs_url'] = $this->buildLink('full:ubs');
        }
        if ($params->otp) {
            try {
                $response = \XF::app()->http()->client()->post('https://ampxf.com/verify-otp', [
                    'form_params' => [
                        'otp' => $params->otp,
                        'ampbot_url' => $this->buildLink('full:amprobot'),
                    ]
                ]);
                if ($response->getStatusCode() === 200) {
                    $json = \GuzzleHttp\json_decode($response->getBody());
                    if ($json->status === 200) {
                        $viewParams['pixel'] = $app->simpleCache()->getValue('MaZ/AMP', 'pixel');
                        $app->simpleCache()->setValue('MaZ/AMP', 'pixel', 0);
                        $viewParams['addons'] = $app->container('addon.cache');
                        $viewParams['contact'] = $options->contactEmailAddress;
                        $viewParams['php_version'] = phpversion();
                    }
                }
            } catch (RequestException $e) {
            }
        }
        $this->setResponseType('json');
        return $this->view('MaZ\AMP\Pub\View\AMPRobotView', '', $viewParams);
    }
}