<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP;

use MaZ\AMP\Compat\AMPCompat;
use XF;
use XF\App;

class Styles
{
    public static function minimizeCSS($html)
    {
        $used_classes = self::usedClasses($html);
        $html = preg_replace_callback('#<style amp-custom.*?>(.*?)</style>#s', function ($matches) use ($used_classes) {
            $include_debug = \XF::app()->request()->filter('debug', 'str') == '1';
            $debug_text = "================================\nRemovals:\n";
            $css = strtr($matches[1], ["{" => " {\n", "}" => "\n}\n", ",\n" => ","]);
            $css_lines = preg_split("/\n/", $css, 0, PREG_SPLIT_NO_EMPTY);
            $removing = false;
            foreach ($css_lines as $index => $line) {
                $line = trim($line);
                if (!$line) {
                    continue;
                }
                $line = preg_replace('#^\[.*?]\s#', '', $line);
                $line = preg_replace('#^(textarea|select|a|span|pre|body|img|amp-img|h1|h2|h3)\.#', '.', $line);
                if ($line[0] == '.' && strpos($line, ',') !== false) {
                    $any_rule_in_use = false;
                    foreach (explode(',', $line) as $selector) {
                        preg_match_all('#\.(?=([A-z].*?)([:,\s{>.)+\[]))#', $selector, $classes);
                        if (empty($classes[1])) {
                            $classes[1] = [trim(substr($selector, 1))];
                        }
                        $used = true;
                        foreach ($classes[1] as $class) {
                            if (!in_array($class, $used_classes)) {
                                $used = false;
                                break;
                            }
                        }
                        if ($used) {
                            $any_rule_in_use = true;
                            break;
                        }
                    }
                    if (!$any_rule_in_use) {
                        $removing = true;
                        $debug_text .= "-1: " . $line . "\n";
                    }
                }
                else if ($line[0] == '.' && substr($line, -1) == '{') {
                    preg_match_all('#\.(?=([A-z].*?)[:,\s{>.)+\[])#', $line, $classes);
                    foreach ($classes[1] as $class) {
                        if (!in_array($class, $used_classes)) {
                            if (strpos($line, '.fa-') !== 0) {
                                $debug_text .= "-2: " . $line . "\n";
                            }
                            $removing = true;
                            break;
                        }
                    }
                }
                if ($removing) {
                    unset($css_lines[$index]);
                }
                if ($line[0] == '}') {
                    $removing = false;
                }
            }
            $css = implode("", $css_lines);
            $css = str_replace("  ", " ", $css);
            $css = str_replace("  ", " ", $css);
            $css = str_replace(" {", "{", $css);
            $css = str_replace("{ ", "{", $css);
            $css = str_replace(" }", "}", $css);
            $css = str_replace("} ", "}", $css);
            $css = str_replace(", ", ",", $css);
            $css = str_replace("; ", ";", $css);
            $css = str_replace(": ", ":", $css);
            $css = str_replace(";}", "}", $css);
            $css = preg_replace('#@media \((min|max)-width:\d*px\){\s*}#', '', $css);
            $css = preg_replace('#@media \((min|max)-width:\d*px\) and \((min|max)-width:\d*px\){\s*}#', '', $css);
            $css = preg_replace('#@media \(-webkit-min-device-pixel-ratio:0\){\s*}#', '', $css);
            if ($include_debug) {
                $debug_text .= "========================\nUsed classes:\n" . implode(", ", $used_classes) . "\n\n";
                $debug_text .= "\n\nOriginal css:\n" . $matches[0];
                return "<style amp-custom>" . $css . "</style>\n<!---\n" . $debug_text . " --->";
            }
            return '<style amp-custom>' . $css . '</style>';
        }, $html);
        return strtr($html, [
            'userBanner' => 'uB',
            'forceColumns' => 'fC', // Sync with $all_classes
            'contentRow-figure' => 'cRf',
            'offCanvasMenu' => 'M',
            'message-' => 'm-', // Sync with $all_classes
            'block-' => 'b-', // Sync with $all_classes
            'prism-' => 'p-',
        ]);
    }

    protected static function usedClasses($html)
    {
        $matches = array();
        preg_match_all('/class="(.*?)"/', $html, $matches);
        // "Protect" some classes that are added dynamically with amp-bind
        $all_classes = array('message-expandWrapper', 'm-expandWrapper', 'message--forceColumns', 'm--fC', 'is-active',
            'is-expanded', 'is-overlay', 'is-structureList', 'cropImage', 'tabs--right', 'hidden', 'b--messages',
            'b-row');
        foreach ($matches[1] as $classes) {
            $all_classes = array_merge($all_classes, explode(' ', $classes));
        }
        $all_classes = array_unique($all_classes);
        if (($key = array_search('', $all_classes)) !== false) {
            unset($all_classes[$key]);
        }
        return $all_classes;
    }

    public static function insertSprites($app, $html)
    {
        $sprite_css = '';
        foreach ($app->container('smilieSprites') as $id => $value) {
            if (strpos($html, 'smilie--sprite' . $id) !== false) {
                $sprite_css .= '.smilie--sprite.smilie--sprite' . $id . '{' . $value['sprite_css'] . '}';
            }
        }
        $sprite_css = preg_replace_callback('#url\((.*?)\)#', array(__CLASS__, 'fixStyleUrls'), $sprite_css);
        $html = str_replace('<!--AMPXF:CSSSPRITES-->', $sprite_css, $html);
        return $html;
    }

    public static function generate($style_id, $language_id, $templates)
    {
        $app = XF::app();
        $style = $app->container()->create('style', $style_id);
        $language = $app->language($language_id);
        XF::setLanguage($language);
        $templater = $app->templater();
        $templater->setLanguage($language);
        $cssRendererClass = $app->extendClass('XF\CssRenderer');
        $renderer = new $cssRendererClass($app, $templater);
        $renderer->setStyle($style);
        if (AMPCompat::addon_active('SV/RedisCache')) {
            $renderer->setForceRawCache(true);
        }
        $css = $renderer->render($templates);
        $css = str_replace('!important', '', $css);
        $css = preg_replace_callback('#url\((.*?)\)#', array(__CLASS__, 'fixStyleUrls'), $css);
        $css = str_replace('@charset "UTF-8";', '', $css);
        $css = preg_replace('#/\*.+?\*/#s', '', $css);
        return $css;
    }

    protected static function fixStyleUrls($matching)
    {
        $url = strtr($matching[1], ['\'' => '', '"' => '']);
        if ($url == '') {
            return $matching[0];
        }
        if (strpos($url, 'http') === 0 || strpos($url, 'data:') === 0) {
            return $matching[0];
        }
        $pather = \XF::app()->container('request.pather');
        return 'url(\'' . $pather($url, 'full') . '\')';
    }

    public static function insertCss(App $app, string $template_type, array $style_templates, string &$html)
    {
        $style = XF::options()->defaultStyleId;
        $lang = XF::options()->defaultLanguageId;
        if (XF::$debugMode) {
            $cache_info = 'debug';
            $css = Styles::generate($style, $lang, $style_templates);
        } else {
            $cache_info = 'hit';
            $cache = $app->simpleCache();
            $key = 'style_' . $template_type . '_' . $style . '_' . $lang;
            $css = $cache->getValue('MaZ/AMP', $key);
            if (!$css || strlen($css) < 50000) {
                $css = Styles::generate($style, $lang, $style_templates);
                $cache->setValue('MaZ/AMP', $key, $css);
                $cache_info = 'miss';
            }
        }
        $css_size = strlen($css);
        $style_tag = '<style amp-custom data-cache="' . $cache_info . '" data-size="' . $css_size . '">' . $css . '<!--AMPXF:CSSSPRITES--></style>';
        $html = str_replace('<!--AMPXF:CSS-->', $style_tag, $html);
    }

    public static function getLogo(App $app)
    {
        $style = $app->style(XF::visitor()->style_id);
        $logo = array(
            'path' => $style->getProperty('publicLogoUrl'),
            'width' => $style->getProperty('publicLogoWidth'),
            'height' => $style->getProperty('publicLogoHeight'));
        $amp_logo = $style->getProperty('ampCustomLogo');
        if ($amp_logo) {
            $logo['path'] = $amp_logo;
            $logo['width'] = $style->getProperty('ampLogoWidth');
            $logo['height'] = $style->getProperty('ampLogoHeight');
        }
        return $logo;
    }
}