<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\XF\Pub\Controller;

use MaZ\AMP\Compat\AMPCompat;
use XF\Mvc\Reply\View;
use XF\Mvc\ParameterBag;

class Forum extends XFCP_Forum
{
    public function actionList(ParameterBag $params)
    {
        if (\XF::options()->ampDisableForum) {
            if ($this->request->filter('amp', 'str') == '1') {
                return $this->redirect(substr($this->request->getFullRequestUri(), 0, -6));
            }
            return parent::actionList($params);
        }
        if (empty($params->node_id) && empty($params->node_name) && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'forumhomepixel', $params);
        }
        $reply = parent::actionList($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $forums_url = $this->buildLink('full:forums');
            if ($this->request->filter('amp', 'str') == '1') {
                AMPCompat::set_amp_page_params($reply, $templater, $forums_url);
            } else {
                AMPCompat::set_meta_amp_url($templater, $forums_url);
            }
        }
        return $reply;
    }

    public function actionForum(ParameterBag $params)
    {
        if (\XF::options()->ampDisableForum) {
            if ($this->request->filter('amp', 'str') == '1') {
                return $this->redirect(substr($this->request->getFullRequestUri(), 0, -6));
            }
            return parent::actionForum($params);
        }
        if ($params->node_id && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'forumpixel', $params);
        }
        $reply = parent::actionForum($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $forum = $reply->getParam('forum');
            $page = $reply->getParam('page');
            $forums_url = $this->buildLink('full:forums', $forum, ['page' => $page]);
            if ($this->request->filter('amp', 'str') == '1') {
                AMPCompat::set_amp_page_params($reply, $templater, $forums_url);
                if (in_array($params->node_id, \XF::options()->skipAdsForNodeIds)) {
                    $templater->setPageParam('ampNodeDisableAds', true);
                }
            } else {
                AMPCompat::set_meta_amp_url($templater, $forums_url);
            }
        }
        return $reply;
    }

    public function actionForumHomePixel(ParameterBag $params)
    {
        $selfRoute = ($this->options()->forumsDefaultPage == 'forums' ? 'forums' : 'forums/list');
        $this->assertCanonicalUrl($this->buildLink($selfRoute));
        $cache = $this->app->simpleCache()->getSet('MaZ/AMP');
        $cache->setValue('pixel', $cache->getValue('pixel') + 1);
        return $this->view('MaZ\AMP\Pub\View\AMPPixelView', '');
    }

    public function actionForumPixel(ParameterBag $params)
    {
        $this->assertViewableForum($params->node_id ?: $params->node_name, $this->getForumViewExtraWith());
        $cache = $this->app->simpleCache()->getSet('MaZ/AMP');
        $cache->setValue('pixel', $cache->getValue('pixel') + 1);
        return $this->view('MaZ\AMP\Pub\View\AMPPixelView', '');
    }
}
