<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\XF\Pub\Controller;

use MaZ\AMP\Compat\AMPCompat;
use XF\Mvc\Reply\View;
use XF\Mvc\ParameterBag;

class Thread extends XFCP_Thread
{
    public function actionIndex(ParameterBag $params)
    {
        if ($params->thread_id && $this->request->filter('amp-pixel', 'str') == '1') {
            return $this->rerouteController(__CLASS__, 'pixel', $params);
        }
        $reply = parent::actionIndex($params);
        if ($reply instanceof View) {
            $templater = $this->app->templater();
            $thread = $reply->getParam('thread');
            $thread_url = $this->buildLink('full:threads', $thread, ['page' => $params->page]);
            $amp_qs = $this->request->filter('amp', 'str') == '1';
            if (in_array($thread->user_id, \XF::options()->ampDisableForThreadStarter)) {
                if ($amp_qs) {
                    return $this->redirect($thread_url);
                }
                return $reply;
            }
            if ($amp_qs) {
                AMPCompat::set_amp_page_params($reply, $templater, $thread_url);
                if (\XF::$versionId >= 2020000) {
                    $templater->setPageParam('ampNoIndex', !$thread->isSearchEngineIndexable());
                }
                $no_ads_threadids = explode(',', \XF::options()->ampSkipAdsForThreadIds);
                if (in_array($thread->node_id, \XF::options()->skipAdsForNodeIds) || in_array($thread->thread_id, $no_ads_threadids)) {
                    $templater->setPageParam('ampNodeDisableAds', true);
                }
            } else if (\XF::$versionId < 2020000 || $thread->isSearchEngineIndexable()) {
                AMPCompat::set_meta_amp_url($templater, $thread_url);
            }
        }
        return $reply;
    }

    public function actionPixel(ParameterBag $params)
    {
        $this->assertViewableThread($params->thread_id, $this->getThreadViewExtraWith());
        $cache = $this->app->simpleCache()->getSet('MaZ/AMP');
        $cache->setValue('pixel', $cache->getValue('pixel') + 1);
        return $this->view('MaZ\AMP\Pub\View\AMPPixelView', '');
    }
}
