<?php
/************************************************************************\
 *
 * AMPXF
 *
 * Copyright ©2020 JEAW Ab. All Rights Reserved.
 * This file may not be redistributed in whole or significant part.
 *
 * AMPXF IS NOT FREE SOFTWARE
 * https://ampxf.com/
 *
 * \************************************************************************/

namespace MaZ\AMP\XF\Template;

use XF\App;
use XF\Language;
use XF\Template\MacroState;

class Templater extends XFCP_Templater
{
    public function __construct(App $app, Language $language, $compiledPath)
    {
        $this->defaultFunctions['amp_fa'] = 'fnAMPFAIcon';
        parent::__construct($app, $language, $compiledPath);
    }

    public function fnAMPFAIcon($templater, &$escape, $icon = null, $full = false)
    {
        return $this->preEscaped('<i class="fa--xf far ' . $icon . '" aria-hidden="true"></i>', 'html');
    }

    protected function isAMP()
    {
        return $this->app->request()->filter('amp', 'str') == '1';
    }

    public function fnAvatar($templater, &$escape, $user, $size, $canonical = false, $attributes = [])
    {
        $html = parent::fnAvatar($templater, $escape, $user, $size, $canonical, $attributes);
        if (self::isAMP()) {
            $html = str_replace('<img ', '<amp-img layout="fill" ', $html);
            $html = str_replace(' />', "></amp-img>", $html);
            $html = str_replace('amp="true"', "", $html);
            $html = str_replace(' loading="lazy"', "", $html);
        }
        return $html;
    }

    public function fnReaction($templater, &$escape, array $config)
    {
        $html = parent::fnReaction($templater, $escape, $config);
        if (self::isAMP()) {
            $pxSize = 21;
            if ($config['small']) {
                $pxSize = 16;
            }
            $html = str_replace('<img ', '<amp-img width="' . $pxSize . '" height="' . $pxSize . '" ', $html);
            $html = str_replace(' />', "></amp-img>", $html);
        }
        return $html;
    }


    private function get_title($globalVars)
    {
        if (isset($globalVars['thread']->title)) {
            return $globalVars['thread']->title;
        } else if (isset($globalVars['resource']->title)) {
            return $globalVars['resource']->title;
        } else if (isset($globalVars['mediaItem']->title)) {
            return $globalVars['mediaItem']->title;
        } else if (isset($globalVars['blogEntry']->title)) {
            return $globalVars['blogEntry']->title;
        } else if (isset($globalVars['article']->title)) {
            return $globalVars['article']->title;
        } else if (isset($globalVars['item']->title)) {
            return $globalVars['item']->title;
        }
        return '';
    }

    public function callMacro($template, $name, array $arguments, array $globalVars, MacroState $macroState = null)
    {
        if (self::isAMP()) {
            if ($name == 'page_jump_menu') {
                return '';
            }
            if ($template == 'attachment_macros' && self::isAMP() && \XF::$versionId >= 2020000) {
                $template = 'amp_' . $template;
            }
            if ($template == 'share_page_macros' && $name == 'buttons') {
                $arguments['pageUrl'] = urlencode(substr($globalVars['xf']['fullUri'], 0, -6));
                $arguments['pageTitle'] = urlencode(self::get_title($globalVars));
                $template = 'amp_' . $template;
            }
            $html = parent::callMacro($template, $name, $arguments, $globalVars, $macroState);
            if ($template == 'content_vote_macros' && strpos($html, 'href="javascript:"') !== false) {
                $html = str_replace('href="javascript:"', 'href="#"', $html);
            }
            if ($name == 'post_attribution') {
                $html = preg_replace_callback('#<a href="(.*?)".*?</a>#s', function ($matches) {
                    if (strpos($matches[0], 'share-tooltip') !== false) {
                        $fa_weight = \XF::app()->style(\XF::visitor()->style_id)->getProperty('fontAwesomeWeight');
                        $fa_style = 'far';
                        if ($fa_weight == 300) {
                            $fa_style = 'fal';
                        } elseif ($fa_weight == 900) {
                            $fa_style = 'fas';
                        }
                        $url = \XF::options()->boardUrl . $matches[1];
                        return '<span class="ampshare"><amp-social-share type="system" layout="fill" data-param-url="' . $url . '"><i class="' . $fa_style . ' fa-share-alt"></i></amp-social-share></span>';
                    }
                    return $matches[0];
                }, $html);
            }
            if ($name == 'post_article_macros::article' && strpos($html, 'is-expanded') === false) {
                $this->app->templater()->setPageParam('ampBind', true);
                $html = str_replace('class="message-expandWrapper', '[class]="expanded ? \'message-expandWrapper is-expanded\' : 0" class="message-expandWrapper is-expandable', $html);
                $html = str_replace('class="message-expandLink', 'role="button" tabindex="0" on="tap:AMP.setState({expanded: true})" class="message-expandLink', $html);
            }
            if ($name == 'notice' && strpos($html, 'href="/account/dismiss') !== false) {
                preg_match('#data-notice-id="(\d+)"#s', $html, $ids);
                $html = str_replace('data-notice-id="', 'id="notice', $html);
                $html = str_replace('class="notice-dismiss', 'role="button" tabindex="0" on="tap:notice' . $ids[1] . '.hide" class="notice-dismiss', $html);
                $html = str_replace('href="/account/dismiss', 'href="#/account/dismiss', $html);
            }
            return $html;
        }
        return parent::callMacro($template, $name, $arguments, $globalVars, $macroState);
    }

    protected $amp_ad_block_container = array(
        'container_breadcrumb_top_above_amp', 'container_breadcrumb_top_below_amp',
        'container_content_above_amp', 'container_content_below_amp',
        'container_breadcrumb_bottom_above_amp', 'container_breadcrumb_bottom_below_amp',
        'post_below_container_amp',
    );

    public function callAdsMacro($position, array $arguments, array $globalVars)
    {
        if (self::isAMP()) {
            if (!empty($this->app->templater()->pageParams['ampNodeDisableAds'])) {
                return '';
            }
            if (strpos($position, '_amp') === false) {
                $position = $position . '_amp';
            }
            $html = parent::callAdsMacro($position, $arguments, $globalVars);
            if ($html && strlen($html) > 30 && in_array($position, $this->amp_ad_block_container)) {
                $html = '<div class="block-container block-container-ad">' . $html . '</div>';
            }
            return $html;
        }
        return parent::callAdsMacro($position, $arguments, $globalVars);
    }

    public function form($contentHtml, array $options)
    {
        $html = parent::form($contentHtml, $options);
        if (self::isAMP()) {
            $html = str_replace('<form action=', '<form action-xhr=', $html);
        }
        return $html;
    }

}
